@@ -586,22 +586,28 @@ class Plot2D:
             p1, p2 = self.calculate_p1p2(direction, cell_number)
             resx = self.model._grid.regular_grid.resolution[0]
             resy = self.model._grid.regular_grid.resolution[1]
-            x, y, z = self._slice_topo_4_sections(p1, p2, resx)
-            if direction == 'x':
-                a = np.vstack((y, z)).T
-                ext = self.model._grid.regular_grid.extent[[2, 3]]
-            elif direction == 'y':
-                a = np.vstack((x, z)).T
-                ext = self.model._grid.regular_grid.extent[[0, 1]]
-            else:
-                raise NotImplementedError
-            a = np.append(a,
-                          ([ext[1], a[:, 1][-1]],
-                           [ext[1], self.model._grid.regular_grid.extent[5]],
-                           [ext[0], self.model._grid.regular_grid.extent[5]],
-                           [ext[0], a[:, 1][0]]))
-            line = a.reshape(-1, 2)
-            ax.fill(line[:, 0], line[:, 1], color='k')
+
+            try:
+                x, y, z = self._slice_topo_4_sections(p1, p2, resx)
+                if direction == 'x':
+                    a = np.vstack((y, z)).T
+                    ext = self.model._grid.regular_grid.extent[[2, 3]]
+                elif direction == 'y':
+                    a = np.vstack((x, z)).T
+                    ext = self.model._grid.regular_grid.extent[[0, 1]]
+                else:
+                    raise NotImplementedError
+                a = np.append(a,
+                              ([ext[1], a[:, 1][-1]],
+                               [ext[1], self.model._grid.regular_grid.extent[5]],
+                               [ext[0], self.model._grid.regular_grid.extent[5]],
+                               [ext[0], a[:, 1][0]]))
+                line = a.reshape(-1, 2)
+                ax.fill(line[:, 0], line[:, 1], color='k')
+            except IndexError:
+                warnings.warn('Topography needs to be a raster to be able to plot it'
+                                     'in 2D sections')
+        return ax
 
     def plot_contacts(self, ax, section_name=None, cell_number=None, direction='y', block=None,
                       only_faults=False, **kwargs):
@@ -644,6 +650,7 @@ class Plot2D:
                 for e, block in enumerate(scalar_fields):
                     level = self.model.solutions.scalar_field_at_surface_points[e][np.where(
                         self.model.solutions.scalar_field_at_surface_points[e] != 0)]
+
                     # Ignore warning about some scalars not being on the plot since it is very common
                     # that an interface does not exit for a given section
                     c_id2 = c_id + len(level)  # color id endpoint
